//
//  It is file 'byteorder.cc'
//
//  Copyright (C) 1992, 1993 Serge Pachkovsky, ps@che.nsk.su
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 1, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
#include "stdinc.hpp"
#include "byteorder.hpp"

const unsigned short		word_byte_order  = 0x0100 ;
const unsigned long		dword_byte_order = 0x03020100 ;
const unsigned char * const	word_template	 = (const unsigned char *) (const void *) &word_byte_order ;
const unsigned char * const	dword_template	 = (const unsigned char *) (const void *) &dword_byte_order ;

static void
word_intel_to_native( unsigned char *data )
{
    unsigned char   buf[ 2 ] ;

buf[ 0 ] = data[ word_template[ 0 ] ] ;
buf[ 1 ] = data[ word_template[ 1 ] ] ;
memcpy( data, buf, 2 ) ;
}

static void
word_native_to_intel( unsigned char *data )
{
    unsigned char   buf[ 2 ] ;

buf[ word_template[ 0 ] ] = data[ 0 ] ;
buf[ word_template[ 1 ] ] = data[ 1 ] ;
memcpy( data, buf, 2 ) ;
}

static void
dword_intel_to_native( unsigned char *data )
{
    unsigned char   buf[ 4 ] ;

buf[ 0 ] = data[ dword_template[ 0 ] ] ;
buf[ 1 ] = data[ dword_template[ 1 ] ] ;
buf[ 2 ] = data[ dword_template[ 2 ] ] ;
buf[ 3 ] = data[ dword_template[ 3 ] ] ;
memcpy( data, buf, 4 ) ;
}

static void
dword_native_to_intel( unsigned char *data )
{
    unsigned char   buf[ 4 ] ;

buf[ dword_template[ 0 ] ] = data[ 0 ] ;
buf[ dword_template[ 1 ] ] = data[ 1 ] ;
buf[ dword_template[ 2 ] ] = data[ 2 ] ;
buf[ dword_template[ 3 ] ] = data[ 3 ] ;
memcpy( data, buf, 4 ) ;
}

void
ToNative( void *data, ByteOrder patt )
{
    unsigned char *	s = (unsigned char *) data ;
    ByteOrderEntry *	p = patt ;

for( ; p->Type != ConvertStop ; s += p++->FieldWidth )
    switch( p->Type ){
	default:
	    cerr << "ToNative(): invalid conversion template\n" ;
	    exit( -1 ) ;
	case ConvertNone:
	    break ;
	case ConvertWord:
	    word_intel_to_native( s ) ;
	    break ;
	case ConvertDword:
	    dword_intel_to_native( s ) ;
	    break ;
	}
}

void
ToIntel( void *data, ByteOrder patt )
{
    unsigned char *	s = (unsigned char *) data ;
    ByteOrderEntry *	p = patt ;

for( ; p->Type != ConvertStop ; s += p++->FieldWidth )
    switch( p->Type ){
	default:
	    cerr << "ToIntel(): invalid conversion template\n" ;
	    exit( -1 ) ;
	case ConvertNone:
	    break ;
	case ConvertWord:
	    word_native_to_intel( s ) ;
	    break ;
	case ConvertDword:
	    dword_intel_to_native( s ) ;
	    break ;
	}
}
