//
//  It is file 'con_dcl.hpp'
//
//  Copyright (C) 1992, 1993 Serge Pachkovsky, ps@che.nsk.su
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 1, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
struct Stats {
    CharType	    symbol ;
    CountType	    count ;
    } ;

struct FStats {
    CountType	    count ;
    } ;

struct Links {
    class Context  *next ;
    } ;

const	int	MaxLooseLeafLinks = ( sizeof( FStats ) * CharsCount ) / sizeof( Stats ) ;
const	int	MaxLooseNodeLinks = ( ( sizeof( FStats ) + sizeof( Links ) ) * CharsCount ) /
					( sizeof( Stats ) + sizeof( Links ) ) ;
const	char	ContextLeaf	 = 1 ;
const	char	ContextFast	 = 2 ;
const	char	ContextBurstable = 4 ;
const	char	ContextAllFlags  = ContextLeaf | ContextFast | ContextBurstable ;

const	INT16	NoBurstChar = -1 ;

enum ContextType { Leaf, Node } ;

class Context {
    union {
	Stats	   *stats ;
	FStats	   *fstats ;
	} ;
    Links	   *links ;
    unsigned long   use_count ;
    INT16	    max_index ;
    #ifdef CODE_BURST
    INT16	    burst_char ;
    #endif
    char	    rescale_count ;
    char	    flags ;
    #ifdef DEBUG
    public:
    CharType	    context_id[ MaxModelOrder + 1 ] ;
    int 	    context_level ;
    void	    SetContextId( int, Context *, CharType ) ;
    private:
    #endif
    Boolean	    FastStats( void ) const ;
    void	    ConvertToOrdinaryArray( void ) ;
    void	    ConvertToLooseArray( void ) ;
    Boolean	    GoodForLoose( int size ) ;
    int 	    CreateIndex( CharType, int ) ;
    public:
		    Context( ContextType = Node ) ;
		   ~Context( void ) ;
    //
    //	Utilities
    //
    Boolean	    IsLeaf( void ) const ;
    Boolean	    IsBurstable( void ) const ;
    void	    SetBurstableFlag( void ) ;
    void	    RescaleCharCounts( void ) ;
    void	    IncrementUseCount( void ) ;
    unsigned long   GetUseCount( void ) ;
    void	    ScaleUseCount( void ) ;
    void	    DeleteLeastUsed( void ) ;
    void	    KillEmptyLinks( void ) ;
    int 	    GetRescaleCount( void ) const ;
    #ifdef DEBUG
    void	    CheckSanity( const char *from = "Somewhere" ) const ;
    void	    DumpContextInfo( void ) const ;
    #endif
    void *	    operator new( size_t size ) ;
    void	    operator delete( void * p ) ;
    //
    //	Loose-array access routines
    //
    Context	   *GetNext( CharType ) const ;
    void	    SetNext( CharType, Context * ) ;
    CountType	    GetCount( CharType ) const ;
    void	    IncrementCharCount( CharType ) ;
    //
    //	Low-level access routines
    //
    int 	    GetMaxIndex( void ) const ;
    int 	    MakeIndex( CharType ) ;
    int 	    CharToIndex( CharType ) const ;
    CharType	    IndexToChar( int ) const ;
    CountType	    GetCount( int ) const ;
    void	    SetCount( int, CountType ) ;
    Context	   *GetNext( int ) const ;
    void	    SetNext( int, Context * ) ;
    } ;

